local addonName, addon = ...
addon.L = addon.L or {} --setup localization
local L = setmetatable(addon.L, {__index = function(t,i) return i end} )

local colorMap = {
	["1"] = "|cff000000"..L.Black,
	["2"] = "|cff0000ff"..L.Blue,
	["3"] = "|cff00ff00"..L.Green,
	["4"] = "|cffff00ff"..L.Purple,
	["5"] = "|cffff0000"..L.Red,
	["6"] = "|cffffff00"..L.Yellow,
}

local defaults = {
	global = {
		["enabled"] = true,
		["announces"] = {
			raid = false,
			raidWarning = true,
			privateRaidWarning = false,
			privateChatMessage = false,
		},
		[3] = { --Normal mode combos:			
			["126"] = L.Yellow, --Black, Blue, Yellow
			["135"] = L.Green, --Black, Green, Red
			["145"] = L.Purple, --Black, Purple, Red
			["234"] = L.Purple, --Blue, Green, Purple
			["246"] = L.Purple, --Blue, Purple, Yellow
			["356"] = L.Green, --Green, Red, Yellow
		},
		[4] = { --Heroic mode combos (don't know which are actually possible so just listing them all, and priority is just a best-guess):
			["1234"] = L.Purple, --Black, Blue, Green, Purple
			["1235"] = L.Green, --Black, Blue, Green, Red
			["1236"] = L.Green, --Black, Blue, Green, Yellow
			["1245"] = L.Purple, --Black, Blue, Purple, Red
			["1246"] = L.Purple, --Black, Blue, Purple, Yellow
			["1256"] = L.Yellow, --Black, Blue, Red, Yellow
			["1345"] = L.Green, --Black, Green, Purple, Red
			["1346"] = L.Purple, --Black, Gree, Purple, Yellow
			["1356"] = L.Green, --Black, Green, Red, Yellow
			["1456"] = L.Purple, --Black, Purple, Red, Yellow
			["2345"] = L.Green, --Blue, Green, Purple, Red
			["2346"] = L.Purple, --Blue, Green, Purple, Yellow
			["2356"] = L.Green, --Blue, Green, Red, Yellow
			["2456"] = L.Purple, --Blue, Purple, Red, Yellow
			["3456"] = L.Green, --Green, Purple, Red, Yellow
		},
	},
}

local optionsTable = {
    name = addonName,
    handler = addon,
    type = 'group',
    args = {
		enable = {
			name = L.Enable,
			type = 'toggle',
			set = function(_, enabled)
				addon.db.global.enabled = enabled
				if enabled then
					addon:Enable()
				else
					addon:Disable()
				end
			end,
			get = function() return addon.db.global.enabled end,
		},
		reset = {
			name = L['Restore Defaults'],
			desc = L['Restore all combinations to default kill targets.'],
			type = 'execute',
			func = function()
				addon.db:ResetDB()
			end,
		},
		normal = {
			type = 'group',
			name = L['Normal Mode'],
			desc = L['Setup priorities for normal mode combinations.'],
			order = 1,
			args = {
				Header = {
					type = 'header',
					name = L['Normal Mode Globule Combinations'],
					order = 0,
				},
			},
		},
		heroic = {
			type = 'group',
			name = L['Heroic Mode'],
			desc = L['Setup priorities for heroic mode combinations.'],
			order = 2,
			args = {
				Header = {
					type = 'header',
					name = L['Heroic Mode Globule Combinations'],
					order = 0,
				},
				Disclaimer = {
					type = 'description',
					order = 1,
					name = L['If heroic mode is like normal mode, only some of these combinations are actually possible. Unlike normal mode, I do not yet know which are/aren\'t possible, so I have included all 15 possible combinations. Globule detection in heroic mode should work, but is completely untested.'],
				},
			},
		},
		output = {
			type = 'group',
			name = L.Output,
			desc = L['Setup where kill target messages are sent.'],
			order = 3,
			args = {
				Header = {
					type = 'header',
					name = L['Output Destinations'],
					order = 0,
				},
				raid = {
					name = L.Raid,
					type = 'toggle',
					set = function(_, enabled)
						addon.db.global.announces.raid = enabled
					end,
					get = function() return addon.db.global.announces.raid end,
				},
				raidWarning = {
					name = L['Raid Warning'],
					type = 'toggle',
					set = function(_, enabled)
						addon.db.global.announces.raidWarning = enabled
					end,
					get = function() return addon.db.global.announces.raidWarning end,
				},
				privateRaidWarning = {
					name = L['Private Raid Warning'],
					type = 'toggle',
					set = function(_, enabled)
						addon.db.global.announces.privateRaidWarning = enabled
					end,
					get = function() return addon.db.global.announces.privateRaidWarning end,
				},
				privateChatMessage = {
					name = L['Private Chat Message'],
					type = 'toggle',
					set = function(_, enabled)
						addon.db.global.announces.privateChatMessage = enabled
					end,
					get = function() return addon.db.global.announces.privateChatMessage end,
				},
			},
		},
    },
}

local function CreateOptionsSetting(numberString)
	local nameString = ""
	for i = 1,numberString:len() do
		nameString = nameString..colorMap[numberString:sub(i,i)].." "
	end
	local optionTable = {
		type = 'input',
		name = nameString,
		set = function(_, value)
			addon.db.global[numberString:len()][numberString] = value
		end,
		get = function() return addon.db.global[numberString:len()][numberString] end,
	}
	return optionTable
end

function addon:SetupDB()
	self.db = LibStub("AceDB-3.0"):New(addonName.."DB", defaults)
end

function addon:SetupOptions()
	for k,v in pairs(defaults.global[3]) do --Create normal mode settings tables
		optionsTable.args.normal.args[k] = CreateOptionsSetting(k)
	end
	for k,v in pairs(defaults.global[4]) do --Create heroic mode settings tables
		optionsTable.args.heroic.args[k] = CreateOptionsSetting(k)
	end
	LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, optionsTable)
	LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName)
end